capture log close
log using rm3ch5-binary-esttestfit, replace text

//  Ch 5: Binary Outcomes - estimation, testing & fit |
//      rm3ch5-binary-esttestfit.do 2014-08-05
//  Regression Models for Categorical Outcomes using Stata, 3rd edition
//  Scott Long & Jeremy Freese

version 13.1
clear all
macro drop _all
program drop _all
set linesize 80
set scheme s2manual

//  5.2 Estimation using logit and probit commands

use binlfp4, clear
codebook lfp k5 k618 agecat wc hc lwg inc, compact
tabulate agecat, missing
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc

* comparing logit and probit

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store Mlogit
probit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store Mprobit
estimates table Mlogit Mprobit, b(%9.3f) t varlabel varwidth(30)

* (advanced) observations predicted perfectly

tab lfp k5
logit lfp k5_1 k5_2 k5_3, or nolog
logit lfp k5_1 k5_2 k5_3, or asis nolog

* exact estimation: very slow so do not run each time
* exlogistic lfp k5_1 k5_2 k5_3, memory(2gb)

//  5.3 Hypothesis testing

* testing individual coefficients

logit lfp k5 i.wc i.hc k618 i.agecat lwg inc, nolog
test k5
di sqrt(r(chi2))
display sqrt(52.57)

test 1.wc

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store Mfull
logit lfp k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store Mnok5
lrtest Mfull Mnok5

* testing multiple coefficients

estimates restore Mfull
test 1.hc 1.wc
test 1.hc = 1.wc
test 2.agecat 3.agecat
testparm i.agecat

logit lfp k5 k618 i.agecat lwg inc, nolog
estimates store Mnowchc
lrtest Mfull Mnowchc

logit lfp, nolog
estimates store Mconstant
lrtest Mfull Mconstant

//  5.4 Predicted probabilities, residuals, and influential observations

* predicted probabilities using predict

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store logit
predict prlogit
label var prlogit "Logit: Pr(lfp | X)"
codebook prlogit, compact

dotplot prlogit, ylabel(0(.2)1, grid gmin gmax)
graph export rm3ch5-predict-dotplot.emf, replace

probit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store probit
predict prprobit
label var prprobit "Probit: Pr(lfp | X)"

pwcorr prlogit prprobit

scatter prlogit prprobit, xlabel(0(.25)1, grid) ylabel(0(.25)1, grid) ///
    msymbol(Oh) aspect(1)
graph export rm3ch5-predict-logitprobit.emf, replace

* residuals and influential observations using predict

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
predict rstd, rstandard
label var rstd "Standardized Residual"
sort inc, stable
generate index = _n
label var index "Observation Number"

graph twoway scatter rstd index, msymbol(Oh) mcolor(black) ///
    xlabel(0(200)800) ylabel(-4(2)4, grid gmin gmax) ///
    yline(0, lcolor(black))
graph export rm3ch5-resids.emf, replace

graph twoway scatter rstd index, ///
    msymbol(none) mlabel(index) mlabposition(0) xlabel(0(200)800) ///
    ylabel(-4(2)4, grid gmin gmax) yline(0, lcolor(black))
graph export rm3ch5-resid-index.emf, replace

list lfp k5 k618 2.agecat 3.agecat wc hc lwg inc in 142, clean
sort rstd
list rstd index k5 agecat wc lwg inc if rstd>1.7 | rstd<-1.7, clean

graph twoway scatter rstd index if (rstd>1.7) | (rstd<-1.7), ///
    msymbol(none) mlabel(k5) mlabposition(0) xlabel(0(200)800) ///
    ylabel(-4(2)4, grid gmin gmax) yline(0, lcolor(black)) ///
    caption("Values indicate # of young children")
graph export rm3ch5-resid-large.emf , replace

predict deltabeta, dbeta
label var deltabeta "Pregibon's influence statistic"
graph twoway scatter deltabeta index, ///
    msymbol(none) mlabel(index) mlabposition(0) ///
    xlabel(0(200)800) xtitle("Observation Number") ///
    ylabel(0(.1).3, grid gmin gmax)
graph export rm3ch5-influence-cook.emf , replace

* Least likely observations
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
leastlikely k5 k618 wc agecat

//  5.5 Measures of fit

* information criteria

quietly logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store model1
quietly logit lfp k5 i.agecat i.wc c.inc##c.inc, nolog
estimates store model2
estimates table model1 model2, b(%9.3f) p(%9.3f) stats(N bic aic r2_p)

estimates restore model1
quietly fitstat, save ic
estimates restore model2
fitstat, diff ic

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estat ic
logit lfp k5 i.agecat i.wc c.inc##c.inc, nolog
estat ic

* pseudo-R2’s

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
quietly fitstat, save
logit lfp k5 i.agecat i.wc c.inc##c.inc, nolog
fitstat, diff

* (advanced) Hosmer-Lemeshow statistic

logit lfp k5 k618 i.agecat i.wc i.hc lwg c.inc, nolog
estat gof, group(10)

quietly logit lfp k5 k618 i.agecat i.wc i.hc lwg c.inc, nolog
capture matrix drop hl
forvalues numgroups = 5(1)15 {
    quietly estat gof, group(`numgroups')
    local rm = r(m)
    matrix r =  r(chi2), r(df), chi2tail(r(df),r(chi2))
    matrix rownames r = "`rm' groups"
    matrix hl = nullmat(hl) \ r
}
matrix colnames hl = chi2 df prob
matlist hl, format(%8.3f)

log close
exit
