capture log close
log using rm3ch9-count, replace text

//  Chapter 9: Count Outcomes | rm3ch9-count.do 2014-08-05
//  Regression Models for Categorical Outcomes using Stata, 3rd edition
//  Scott Long & Jeremy Freese

version 13.1
clear all
macro drop _all
set linesize 80
set scheme s2manual
program drop _all

//  9.1 The Poisson distribution

clear all
set obs 21
gen k = _n - 1
label var k "y = # of events"
gen psn1 = poissonp(0.8, k)
label var psn1 "&mu = 0.8"
gen psn2 = poissonp(1.5, k)
label var psn2 "&mu = 1.5"
gen psn3 = poissonp(2.9, k)
label var psn3 "&mu = 2.9"
gen psn4 = poissonp(10.5, k)
label var psn4 "&mu = 10.5"
graph twoway connected psn1 psn2 psn3 psn4 k, ///
    ytitle("Probability") ylabel(0(.1).5) xlabel(0(2)20) ///
    lwidth(thin thin thin thin) msymbol(O D S T)
graph export rm3ch9-poisson-dist.emf, replace

* fitting the Poisson distribution with the poisson command

use couart4, clear
codebook art female married kid5 mentor phd, compact
tabulate art, missing
sum art, detail

* comparing observed and predicted counts with mgen

poisson art, nolog
mgen, pr(0/9) meanpred stub(psn)
list psnval psnobeq psnpreq in 1/10
label var psnobeq "Observed Proportion"
label var psnpreq "Poisson Prediction"
label var psnval "# of Articles"
graph twoway connected psnobeq psnpreq psnval, ///
    ytitle("Probability") ylabel(0(.1).4) xlabel(0/9) msym(O Th) ///
    lwid(thin thin) msize(*1.4 *1.2) mcol(gs1 gs10) // tweaks to remove from text
graph export rm3ch9-psn-dist.emf, replace

//  9.2 The Poisson regression model

* estimation using poisson

poisson art i.female i.married kid5 phd mentor, nolog

* factor and percent changes in E(y | x)

listcoef female mentor, help
listcoef female mentor, percent help

* marginal effects on E(y | x)

poisson art i.female i.married kid5 phd mentor, nolog
mchange
mchange kid5, amount(one) brief

* interpretation using predicted probabilities

mtable, at(married=(0 1) female=1 kid5=0) atmeans pr(0/5) width(7)
mchange married, at(female=1 kid5=0) atmeans pr(0/5) ///
    stat(from to change p) width(7) brief
quietly mtable if married==1 & female==1, at(kid5=(0/3)) long
mtable if married==1 & female==1, pr(0/5) ///
    at(kid5=(0/3)) atvars(_none) right norownumbers brief

* treating a count independent variable as a factor variable

poisson art i.kid5 i.female i.married phd mentor, nolog
quietly mtable if married==1 & female==1, at(kid5=(0/3)) long
mtable if married==1 & female==1, pr(0/5) at(kid5=(0/3)) ///
    atvars(_none) right norownumbers brief

* predicted probabilities using mgen

mgen if married==1 & female==1, atmeans at(kid5=(0/3)) ///
    stub(Fprm) pr(0) predlabel(Married women)
mgen if married==1 & female==0, atmeans at(kid5=(0/3)) ///
    stub(Mprm) pr(0) predlabel(Married men)
label var Fprmpr0 "Married women"
label var Mprmpr0 "Married men"
label var Mprmkid5  "Number of children"
graph twoway connected Fprmpr0 Mprmpr0 Mprmkid5, ///
    msiz(*1.6 *1.265) lwid(thin thin) mcol(black gs8) ///
    ylabel(0(.1).4) yline(.1 .2 .3) xlabel(0/3) msym(O D) ///
    ytitle("Probability of No Articles")
graph export rm3ch9-prm-prob-menwomen.emf, replace

* comparing observed and predicted counts to evaluate model specification

poisson art i.female i.married kid5 phd mentor, nolog
predict prob0, pr(0)
predict prob1, pr(1)
predict prob2, pr(2)
sum prob0 prob1 prob2
mtable, pr(0/2) brief

mgen, stub(PRM) pr(0/9) meanpred
list PRMval PRMpreq PRMobeq in 1/3, clean
label variable PRMobeq  "Observed"
label variable PRMpreq  "PRM"

poisson art, nolog
mgen, stub(PDF) pr(0/9) meanpred
label var PDFobeq "Observed"
label var PDFpreq "Poisson PDF"
graph twoway connected PDFobeq PDFpreq PRMpreq PRMval, ///
    ytitle("Probability") ylabel(0(.1).4, gmax) xlabel(0/9) msym(O Th Sh)
graph export rm3ch9-psn-dist.emf, replace

* (advanced) exposure time

use couexposure4, clear
poisson totalarts kid5 mentor, nolog irr
poisson totalarts kid5 mentor, nolog irr exposure(profage)
poisson totalarts kid5 mentor, nolog irr offset(lnprofage)

constraint define 1 lnprofage=1
poisson totalarts lnprofage kid5 mentor, nolog constraint(1)

poisson totalarts kid5 mentor, nolog irr offset(lnprofage)

//  9.3 The negative binomial regression model

* negative binomial regression model

use couart4, clear
nbreg art i.female i.married kid5 phd mentor

* comparing the PRM and NBRM using estimates table

qui poisson art i.female i.married kid5 phd mentor
estimates store PRM
qui nbreg art i.female i.married kid5 phd mentor
estimates store NBRM
estimates table PRM NBRM, b(%9.3f) t p(%9.3f) varlabel ///
    drop(lnalpha:_cons) stats(alpha N) eform vsquish

* robust standard errors

quietly poisson art i.female i.married kid5 phd mentor
estimates store PRM
quietly poisson art i.female i.married kid5 phd mentor, vce(robust)
estimates store PRMrobust
quietly nbreg art i.female i.married kid5 phd mentor
estimates store NBRM
quietly nbreg art i.female i.married kid5 phd mentor, vce(robust)
estimates store NBRMrobust
estimates table PRM PRMrobust NBRM NBRMrobust, b(%9.3f) se(%9.4f) p(%9.3f) ///
    varlabel drop(lnalpha:_cons) stats(alpha N) eform vsquish modelwidth(10)

* interpretation using E(y | x)

nbreg art i.female i.married kid5 phd mentor, nolog
listcoef female mentor
listcoef female mentor, percent

poisson art i.female i.married kid5 phd mentor, nolog
quietly mtable, estname(PRM_mu) ci at(mentor=(0(5)20) female=1 phd=4) ///
    atmeans clear dec(2)
nbreg art i.female i.married kid5 phd mentor, nolog
mtable, estname(NBRM_mu) ci at(mentor=(0(5)20) female=1 phd=4) ///
    atmeans right dec(2) noatvars norownumbers brief

nbreg art i.female i.married kid5 phd mentor, nolog
mchange kid5, amount(one) stats(ci) brief

* interpretation using predicted probabilities

poisson art i.female i.married kid5 phd mentor, nolog
estimates store PRM
quietly mtable, rowname(PRM) pr(0/7) atmeans
nbreg art i.female i.married kid5 phd mentor, nolog
estimates store NBRM
mtable, rowname(NBRM) pr(0/7) atmeans below brief decimals(3) width(6)
estimates restore PRM
mgen, atmeans at(mentor=(0(5)50)) stub(PRM) pr(0)
estimates restore NBRM
mgen, atmeans at(mentor=(0(5)50)) stub(NBRM) pr(0)
label var PRMpr0 "PRM"
label var NBRMpr0 "NBRM"
graph twoway ///
    (rarea PRMll0 PRMul0 NBRMmentor, color(gs14)) ///
    (rarea NBRMll0 NBRMul0 NBRMmentor, color(gs14)) ///
    (connected PRMpr0 NBRMmentor, lpattern(dash) msize(zero)) ///
    (connected NBRMpr0 NBRMmentor, lpattern(solid) msize(zero)), ///
    legend(on order(3 4)) ylabel(0(.1).4, gmax) ///
    ytitle("Probability of a zero count")
graph export rm3ch9-nbrm-prm-prob0ci.emf, replace

//  9.4 Models for truncated counts

* zero-truncated model

use couart4, clear
quietly nbreg art i.female i.married kid5 phd mentor, nolog vce(robust)
estimates store NBRM

sum art arttrunc
drop if art==0
sum art arttrunc
tnbreg arttrunc i.female i.married kid5 phd mentor, nolog irr
estimates store ZTNBRM

estimates table NBRM ZTNBRM, b(%9.3f) stats(N) t eform

estimates restore ZTNBRM
listcoef female mentor

* predictions in the estimation sample

predict rate
label var rate "Unconditional rate of productivity"
predict crate, cm
label var rate "Conditional rate of productivity given at least 1 article"
summarize rate crate arttrunc

* interpretation using predicted rates and probabilities

qui mtable, rowname(Unconditional) at(female=1 married=1 kid5=0) atmeans ci
mtable, rowname(Conditional) at(female=1 married=1 kid5=0) atmeans ci ///
    predict(cm) below brief
mtable, at(female=1 married=1 kid5=0) atmeans pr(0/5) ci brief
qui mtable, rowname(Unconditional) at(female=1 married=1 kid5=0) atmeans ///
    pr(1/5)
mtable, rowname(Conditional) at(female=1 married=1 kid5=0) atmeans ///
cpr(1/5) below brief

* 9.5 (advanced) hurdle regression model

use couart4, clear
logit art i.female i.married kid5 phd mentor, nolog or
predict prlogit
est store Hlogit

tnbreg art i.female i.married kid5 phd mentor if art>0, nolog irr
est store Hztnb

est table Hlogit Hztnb, b(%9.3f) eform t

suest Hlogit Hztnb, vce(robust) eform(expB)

* predictions in the sample

estimates restore Hlogit
predict Hprobgt0
label var Hprobgt0 "Pr(y>0)"
gen Hprob0 = 1 - Hprobgt0
label var Hprob0 "Pr(y=0)"
summarize Hprob0 Hprobgt0

estimates restore Hztnb
predict Hcrate, cm
label var Hcrate "Conditional rate"
gen Hrate = Hcrate * Hprobgt0
label var Hrate "Unconditional rate"
sum Hcrate Hrate

forvalues icount = 1/8 {
    predict Hcprob`icount', cpr(`icount')
    label variable Hcprob`icount' "Pr(y=`icount'|y>0)"
    gen Hprob`icount' = Hcprob`icount' * Hprobgt0
    label variable Hprob`icount' "Pr(y=`icount')"
}
sum Hprob*

* predictions at user-specified values

est restore Hlogit
mtable, at(female=1 married=1 kid5=0 phd=4.5) atmeans

local prgt0 = el(r(table),1,1)
est restore Hztnb
mtable, at(female=1 married=1 kid5=0 phd=4.5) atmeans noesample cpr(1/8)
matrix Huncond = `prgt0' * r(table)
matlist Huncond, format(%8.3f) title(Unconditional probabilties) names(col)

est restore Hlogit
mtable, at(female=0 phd=1 mentor=0) atmeans
local prgt0 = el(r(table),1,1)
est restore Hztnb
mtable, at(female=0 phd=1 mentor=0) atmeans noesample cpr(1/8)
matrix Huncond = `prgt0' * r(table)
matlist Huncond, format(%8.3f) title(Unconditional probabilties) names(col)

//  9.6 Zero-inflated count models

* zero-inflated models

sysuse couart4, clear
zinb art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) nolog vce(robust)
estimates store zinb

* interpretation of coefficients

listcoef, help

* interpretation of predicted probabilities

quietly mtable, at(female=0 married=1 kid5=3 phd=3 mentor=10) ///
    atvars(female phd mentor) pr(0/5)
mtable, at(female=1 married=1 kid5=3 phd=1 mentor=0) ///
    atvars(female phd mentor) pr(0/5) below width(7) norownumbers

quietly mtable, at(female=0 married=1 kid5=3 phd=3 mentor=10) ///
    atvars(female phd mentor) predict(pr)
mtable, at(female=1 married=1 kid5=3 phd=1 mentor=0) ///
    atvars(female phd mentor) predict(pr) below norownumbers decimals(4)

* plotting predicted probabilities with mgen

mgen, at(mentor=(0/6)) atmeans pr(0) stub(ZINB) replace
mgen, at(mentor=(0/6)) atmeans predict(pr) stub(ZINB) replace
gen ZINBprcount0 = ZINBprany0 - ZINBprall0
label var ZINBprall0 "Always Zero from Binary Equation"
label var ZINBprcount0 "Sometimes Zero from Count Equation"
label var ZINBprany0 "Zeroes from Both Equations"
label var ZINBmentor "Mentor's Publications"
graph twoway connected ZINBprall0 ZINBprcount0 ZINBprany0 ZINBmentor, ///
    xlabel(0/6) ylabel(0(.1).5, gmax) ///
    ytitle(Probability of Zero) msymbol(Sh Dh O) legend(rows(3))
graph export rm3ch9-zinb-mentor-zeros.emf, replace

//  9.7 Comparisons among count models

* comparing mean probabilities

poisson art i.female i.married kid5 phd mentor, nolog
mgen, stub(PRM) pr(0/9) meanpred

nbreg art i.female i.married kid5 phd mentor, nolog
mgen, stub(NBRM) pr(0/9) meanpred

zip art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) nolog
mgen, stub(ZIP) pr(0/9) meanpred

zinb art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) nolog
mgen, stub(ZINB) pr(0/9) meanpred

label variable PRMobeq  "Observed"
label variable PRMpreq  "PRM"
label variable NBRMpreq "NBRM"
label variable ZIPpreq  "ZIP"
label variable ZINBpreq "ZINB"
graph twoway connected PRMobeq PRMpreq NBRMpreq ZIPpreq ZINBpreq PRMval, ///
    ytitle(Average Predicted Probability) ///
    xlabel(0/9) msymbol(Th Oh Sh O S) ///
    legend(col(3) holes(4)) ///
    msiz(*1.2 *1.4 *1.1 *1.4 *1.1) lwid(thin thin thin thin thin)
graph export rm3ch9-compare-simple.emf, replace

gen obs = PRMobeq
gen dPRM = obs - PRMpreq
label var dPRM "PRM"
gen dNBRM = obs - NBRMpreq
label var dNBRM "NBRM"
gen dZIP = obs - ZIPpreq
label var dZIP "ZIP"
gen dZINB = obs - ZINBpreq
label var dZINB "ZINB"
graph twoway connected dPRM dNBRM dZIP dZINB PRMval, ///
    ytitle(Observed-Predicted) ylabel(-.10(.05).10, grid gmin gmax) ///
    xlabel(0/9) msymbol(Oh Sh O S) ///
    subtitle(Positive deviations indicate more observed counts than predicted, ///
    pos(11)) ///
    msiz(*1.4 *1.1 *1.4 *1.1) lwid(thin thin thin thin thin)
graph export rm3ch9-compare-lambert.emf, replace

* tests to compare count models

quietly zip art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor)
estimates store zip
quietly zinb art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor)
estimates store zinb
lrtest zip zinb, force

* Vuong test of non-nested models

zip art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) vuong nolog
listcoef, help

zinb art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) vuong nolog
listcoef, help

* (advanced) computing the Vuong test with a hurdle model

quietly zinb art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor)
forvalues icount = 0/19 {
    predict ZINB`icount', pr(`icount')
    }

quietly logit art i.female i.married kid5 phd mentor
estimates store Hlogit
quietly tnbreg art i.female i.married kid5 phd mentor if art > 0
estimates store Hztnb

estimates restore Hlogit
predict HRMnot0
label var HRMnot0 "HRM prob of non-zero count"
gen HRM0 = 1 - HRMnot0
label var HRM0 "HRM prob of zero count"

estimates restore Hztnb
forvalues icount = 1/19 {
    predict HRM`icount', cpr(`icount')
    quietly replace HRM`icount' = HRM`icount' * HRMnot0
    label var HRM`icount' "HRM unconditional prob(`icount')"
    }

gen ZINBprobs = .
label var ZINBprobs "ZINB prob of count that was observed"
gen HRMprobs = .
label var HRMprobs "HRM prob of count that was observed"
forvalues icount = 0/19 {
    quietly replace ZINBprobs = ZINB`icount' if art == `icount'
    qui replace HRMprobs = HRM`icount' if art == `icount'
    }
sum ZINBprob HRMprobs
gen mZINB_HRM = ln(ZINBprobs/HRMprobs)
sum mZINB_HRM
display (r(mean)*sqrt(r(N)))/r(sd)

* ZINB v NB vuong: not shown in text

capture drop ZINB*
zinb art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) vuong
forvalues icount = 0/19 {
    predict ZINB`icount', pr(`icount')
}
nbreg art i.female i.married kid5 phd mentor
forvalues icount = 0/19 {
    predict NBRM`icount', pr(`icount')
}
gen ZINBprobs = .
gen NBRMprobs = .
forvalues icount = 0/19 {
    qui replace ZINBprobs = ZINB`icount' if art == `icount'
    qui replace NBRMprobs = NBRM`icount' if art == `icount'
}
sum ZINBprob NBRMprobs
gen mZINB_NBRM = ln(ZINBprobs/NBRMprobs)
sum mZINB_NBRM
display "ZINB vs NBRM: " (r(mean)*sqrt(r(N)))/r(sd)

*  ZIP and ZINB: not shown in text

capture drop ZIP* mZ*
zip art i.female i.married kid5 phd mentor, ///
    inflate(i.female i.married kid5 phd mentor) vuong
forvalues icount = 0/19 {
    predict ZIP`icount', pr(`icount')
}
gen ZIPprobs = .
forvalues icount = 0/19 {
    qui replace ZIPprobs = ZIP`icount' if art == `icount'
}
sum ZINBprob ZIPprobs
gen mZINB_ZIP = ln(ZINBprobs/ZIPprobs)
sum mZINB_ZIP
display "ZINB vs ZIP: " (r(mean)*sqrt(r(N)))/r(sd)

* countfit to compare count models

use couart4, clear
countfit art i.female i.married kid5 phd mentor, inflate(i.mentor i.female)
graph export rm3ch9-countfit-lambert.emf, replace

log close
exit
